from time import time as current_timestamp
from flask import (
    current_app as app,
    render_template,
    request,
    redirect,
    url_for,
    Blueprint,
    abort,
    render_template_string,
    send_file,
    jsonify
)

from CTFd.models import db, Files

from CTFd import utils
from CTFd.utils.migrations import upgrade

from CTFd.utils.decorators import admins_only
from CTFd.utils.user import is_admin
from CTFd.utils.security.auth import logout_user
from CTFd.utils.config import is_setup
from CTFd.schemas.files import FileSchema
from CTFd.utils import (
    config as ctf_config,
    validators,
    uploads,
    user as current_user,
    get_config,
    get_app_config,
    set_config,
)

admin = Blueprint('admin', __name__)

#from CTFd.admin import challenges
#from CTFd.admin import pages
#from CTFd.admin import scoreboard
#from CTFd.admin import statistics
#from CTFd.admin import teams
#from CTFd.admin import users
#from CTFd.admin import submissions
#from CTFd.admin import notifications
from CTFd.plugins.openstack_integration.lanes import *
from CTFd.plugins.openstack_integration.challenge import *
# TO DO:
# Create a new challenge type: openstack challenge
# Create new flag type: openstack flag
# -> receives a per-deployment random seed when openstack challenge is deployed. 
# -> unique per user and deployment
# Determine technique for accessing openstack api that minimizes risk to credentials (probably will have to store on disk)

def format_blocked_path(filepath):
    top_dir = filepath.split("/")[0]
    path = "/files/{0}/<filename>".format(top_dir)
    return path

def load(app):
    #upgrade()
    app.db.create_all()
    CHALLENGE_CLASSES['openstack'] = OpenstackIntegrationChallenge
    
    def blocked_path(filename=""):
        return abort(404)
        #return render_template('page.html', content="<h2 style='text-align:center;'>Can't let you do that, Star Fox.</h2>")
    private_files = OpenstackLaneFiles.query.filter(OpenstackLaneFiles.type=='openstack_lane').with_entities(OpenstackLaneFiles.location).all()
    for f in private_files:
        app.add_url_rule(format_blocked_path(f[0]),'blocked_path', blocked_path)

    @app.route('/lane', methods=['GET'])
    def view_faq():
        return render_template('page.html', content="<h1>FAQ Page</h1>")
   
    @app.route('/admin/openstack_lanes', methods=['GET','POST'])
    @admins_only
    def plugin_config():
        return render_template('page.html', content="<div class='container' id='upload_lanes_container'></div><script src='/plugins/openstack_integration/assets/upload_lanes.js?_ts="+str(current_timestamp())+"'></script>")


    @app.route('/admin/get_lane/<int:lane_id>', methods=['GET'])
    @admins_only
    def get_lane(lane_id):
        return get_lane_contents(lane_id)

    @app.route('/admin/delete_lane/<int:lane_id>', methods=['GET'])
    @admins_only
    def delete_lane(lane_id):
        return jsonify(delete_lane_file(lane_id))


    @app.route('/admin/get_lanes', methods=['GET'])
    @admins_only
    def get_lanes():
        lanes = OpenstackLaneFiles.query.all()
        data = []
        for row in lanes:
            data.append({"id" : row.id, "lane_name" : row.lane_name, "start_file" : row.start_file })
        return jsonify(data)

    @app.route('/admin/upload_lane', methods=['POST'])
    @admins_only
    def upload_lane():
        files = request.files.getlist('file')
        objs = []
        for f in files:
            obj = upload_lane_file(file=f, **request.form.to_dict())
            app.add_url_rule(format_blocked_path(obj["location"]),'blocked_path', blocked_path)
            objs.append(obj)
        return jsonify(objs)

    @app.route('/admin/test', methods=['GET', 'POST'])
    @admins_only
    def test_route():
        return render_template('page.html', content="This is a test admin only route")
    #FLAG_CLASSES['openstack'] = CTFdOpenstackFlag
    register_plugin_assets_directory(app, base_path='/plugins/openstack_integration/assets/')
