from __future__ import division # Use floating point for math calculations
from CTFd.models import db, Files
from CTFd.utils.uploads import get_uploader
from flask import current_app as app
import math, zipfile, os, datetime



class OpenstackLaneFiles(Files):
    __tablename__ = 'openstack_lane'
    __mapper_args__ = {
            'polymorphic_identity': 'openstack_lane'
    }
    id = db.Column(None, db.ForeignKey('files.id'), primary_key=True)
    start_file = db.Column(db.Text)
    lane_name = db.Column(db.Text)

    def __init__(self, *args, **kwargs):
        super(Files, self).__init__(**kwargs)
        self.start_file = kwargs['start_file']
        self.lane_name = kwargs['lane_name']

    def as_dict(self):
               return {c.name: getattr(self, c.name) for c in self.__table__.columns}

    def __repr__(self):
        return "<File type={type} location={location}>".format(type=self.type, location=self.location)

def unzip_file(path,dest):
    zip_ref = zipfile.ZipFile(path, 'r')
    zip_ref.extractall(dest)
    zip_ref.close()


def delete_lane_file(file_id):
    f = OpenstackLaneFiles.query.filter_by(id=file_id).first_or_404()
    db.session.delete(f)
    db.session.commit()
    try:
      full_location = "/".join([f.location,f.start_file])
      uploader = get_uploader()
      uploader.delete(filename=full_location)
    except:
      pass
    return file_id


def get_lane_contents(file_id):
    f = OpenstackLaneFiles.query.filter_by(id=file_id).first_or_404()
    full_location = "/".join([f.location,f.start_file])
    uploader = get_uploader()
    try:
      return uploader.download(full_location)
    except:
      return ""
def upload_lane_file(*args, **kwargs):
    file_obj = kwargs.get('file')
    file_type = 'openstack_lane'
    uploader = get_uploader()
    location = uploader.upload(file_obj=file_obj, filename=file_obj.filename)
    start_file = kwargs.get('start_file')
    if "../" in start_file:
      return False
    lane_name = kwargs.get('lane_name')
    upload_filename = location.split("/")[-1:]

    location = "/".join(location.split("/")[:-1])

    if upload_filename[-4:] == ".zip" :
        full_location = os.path.join(app.config.get('UPLOAD_FOLDER'), location, upload_filename)
        p = os.path.join(app.config.get('UPLOAD_FOLDER'),location)
        unzip_file(full_location,p)
    else: 
        start_file = upload_filename

    model_args = {
        'type' : file_type,
        'location' : location,
        'start_file' : start_file,
        'lane_name' : lane_name,
    }
    model = OpenstackLaneFiles
    file_row = model(**model_args)
    db.session.add(file_row)
    db.session.commit()
    model_args["id"] =  file_row.as_dict()["id"]
    return model_args

